"use strict";
/*!
 * Paradigm UI Web
 * Copyright (c) 2017 Miracle Devs, Inc
 * Licensed under MIT (https://gitlab.com/miracledevs-paradigm/ui-web-angularjs/blob/master/LICENSE)
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var base_service_1 = require("./base.service");
var service_1 = require("../decorators/service");
var angular_service_1 = require("./angular.service");
var paradigm_ui_web_shared_1 = require("@miracledevs/paradigm-ui-web-shared");
var KeyProcessorService = /** @class */ (function (_super) {
    tslib_1.__extends(KeyProcessorService, _super);
    function KeyProcessorService(parse) {
        var _this = _super.call(this) || this;
        _this.parse = parse;
        return _this;
    }
    KeyProcessorService_1 = KeyProcessorService;
    KeyProcessorService.prototype.evaluateKeyActions = function (keyActions, eventType, scope, e) {
        keyActions.where(function (x) { return x.eventType === eventType; }).forEach(function (keyAction) {
            if (e.keyCode === keyAction.keyCode &&
                ((paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.shift)) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.shift) && keyAction.shift && e.shiftKey) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.shift) && !keyAction.shift && !e.shiftKey)) &&
                ((paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.alt)) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.alt) && keyAction.alt && e.altKey) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.alt) && !keyAction.alt && !e.altKey)) &&
                ((paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.ctrl)) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.ctrl) && keyAction.ctrl && e.ctrlKey) ||
                    (!paradigm_ui_web_shared_1.ObjectExtensions.isNull(keyAction.ctrl) && !keyAction.ctrl && !e.ctrlKey))) {
                scope.$apply(function () {
                    keyAction.action(scope, { $event: e });
                });
            }
        });
    };
    KeyProcessorService.prototype.parseActions = function (text) {
        var keyActions = new paradigm_ui_web_shared_1.ArrayList();
        var actionStrings = text.split("|");
        for (var i = 0; i < actionStrings.length; i++) {
            var keyAction = new KeyAction();
            var actionString = actionStrings[i];
            var parts = actionString.split(":");
            if (parts.length !== 2)
                throw new Error("The key-submit expression number " + i + " is incorrect. Should be event(keycode,[shift],[alt],[ctrl]): action");
            var firstParenthesis = parts[0].indexOf("(");
            var lastParenthesis = parts[0].indexOf(")");
            if (firstParenthesis < 0 || lastParenthesis < 0)
                throw new Error("The key-submit expression number " + i + " is incorrect. Should be event(keycode,[shift],[alt],[ctrl]): action");
            var eventType = parts[0].substr(0, firstParenthesis);
            if (eventType !== "keypress" &&
                eventType !== "keydown" &&
                eventType !== "keyup")
                throw new Error("Only keypress, keydown and keyup events are allowed on action strings.");
            var keys = parts[0].substr(firstParenthesis + 1, lastParenthesis - firstParenthesis - 1);
            var parameters = new paradigm_ui_web_shared_1.ArrayList(keys.split(","));
            try {
                keyAction.eventType = eventType;
                keyAction.action = this.parse(parts[1]);
                keyAction.keyCode = parseInt(parameters.get(0));
            }
            catch (error) {
                throw new Error("The key-submit expression number " + i + " is incorrect. Should be event(keycode,[shift],[alt],[ctrl]): action.");
            }
            if (parameters.any(function (x) { return x === "shift"; }))
                keyAction.shift = true;
            else if (parameters.any(function (x) { return x === "!shift"; }))
                keyAction.shift = false;
            if (parameters.any(function (x) { return x === "ctrl"; }))
                keyAction.ctrl = true;
            else if (parameters.any(function (x) { return x === "!ctrl"; }))
                keyAction.ctrl = false;
            if (parameters.any(function (x) { return x === "alt"; }))
                keyAction.alt = true;
            else if (parameters.any(function (x) { return x === "!alt"; }))
                keyAction.alt = false;
            keyActions.add(keyAction);
        }
        return keyActions;
    };
    KeyProcessorService.factory = function (parse) {
        return new KeyProcessorService_1(parse);
    };
    KeyProcessorService = KeyProcessorService_1 = tslib_1.__decorate([
        service_1.Service({ name: "$pd-keyProcessorService", dependencies: [angular_service_1.AngularServices.parse] })
    ], KeyProcessorService);
    return KeyProcessorService;
    var KeyProcessorService_1;
}(base_service_1.ServiceBase));
exports.KeyProcessorService = KeyProcessorService;
var KeyAction = /** @class */ (function () {
    function KeyAction() {
    }
    return KeyAction;
}());
exports.KeyAction = KeyAction;
